// transfunc.cc
// deal with function's f90 attributes
// written by Zheng Gengbin

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#else
#include <strings.h>
#endif

#include "compilerApi.h"

#include "tof77.h"

// function's result name
SgSymbol *resultName=NULL;
// 
SgSymbol *funcName=NULL;

// process function header , delete the f90 attributes , only keep the type declaration.
void proc_func(SgStatement *s)
{
	SgExpression *exp2 = s->expr(1);
	SgExpression *newexp = NULL;
	funcName = s->symbol();  ASSERT(funcName);

	for ( ; exp2 ;  exp2=exp2->rhs() )
	{
		switch (exp2->lhs()->variant())
		{
		case TYPE_OP:
			newexp = new SgExpression(EXPR_LIST,exp2->lhs(),newexp,NULL);
			break;
		case PREFIX_RECURSIVE:
			funcName->removeAttribute(RECURSIVE_BIT);
			break;
		}
	}
	s->setExpression(1, newexp);
	// result name
	if (s->expr(0)) {
		resultName = s->expr(0)->symbol();
		s->setExpression(0, NULL);
	}
	else {
		resultName = NULL;
	}
}

void replace_resultName(SgStatement *s)
{
	if (in_function == on && resultName) {
		s->replaceSymbBySymb(*resultName, *funcName);
	}
}

// contains 2 pass : FUNCTION_HEDR_PASS & FUNCTION_CALL_PASS
void analyse3(SgStatement * s)
{
  switch(s->variant())
  {
    case GLOBAL:
      {
         int org_numberOfChildren=s->numberOfChildrenList1();
         SgStatement * * children=new SgStatement * [org_numberOfChildren];

         int i;

         for(i=0;i<org_numberOfChildren;i++)
             children[i]=s->childList1(i);
         for(i=0;i<org_numberOfChildren;i++)
             analyse3(children[i]);

        delete [] children;

        break;
      };

   case PROG_HEDR:
      { 
         int org_numberOfChildren=s->numberOfChildrenList1();
         SgStatement * * children=new SgStatement * [org_numberOfChildren];

         int i;

         for(i=0;i<org_numberOfChildren;i++)
             children[i]=s->childList1(i); 

         in_main_program=on;

         for(i=0;i<org_numberOfChildren;i++)
             analyse3(children[i]);

         delete [] children;

         in_main_program=off;
  
         break;
       };

  case INTERFACE_STMT:
    {
      SgProcHedrStmt * prochedr = (SgProcHedrStmt *)s;
      int org_numberOfChildren=s->numberOfChildrenList1();
      SgStatement * * children = new SgStatement * [org_numberOfChildren];

      int i;

      for (i=0; i<org_numberOfChildren; i++)
	children[i]=s->childList1(i);

      for (i=0; i<org_numberOfChildren; i++)
	analyse3(children[i]);
      
      delete [] children;

      break;
    }

   case PROC_HEDR:
      {
        SgProcHedrStmt * prochedr=(SgProcHedrStmt *)s;
        int org_numberOfChildren=s->numberOfChildrenList1();
        SgStatement * * children=new SgStatement * [org_numberOfChildren];

        int i;

        for(i=0;i<org_numberOfChildren;i++)
            children[i]=s->childList1(i);

        in_procedure=on;

	 if ( passtype == FUNCTION_HEDR_PASS )
	 	proc_func(s);

        for(i=0;i<org_numberOfChildren;i++)
             analyse3(children[i]);
     
        delete [] children;

        in_procedure=off;

        break;
      };

    case FUNC_HEDR:
      {
         SgFuncHedrStmt * prochedr=(SgFuncHedrStmt *)s;
         int org_numberOfChildren=s->numberOfChildrenList1();
         SgStatement * * children=new SgStatement * [org_numberOfChildren];

         int i;

         for(i=0;i<org_numberOfChildren;i++)
              children[i]=s->childList1(i);

         in_function=on;

	 if ( passtype == FUNCTION_HEDR_PASS )
	 	proc_func(s);

         for(i=0;i<org_numberOfChildren;i++)
             analyse3(children[i]);

         delete [] children;

         in_function=off;

         break;
      };  
    case FORALL_STMT:
      {
         int org_numberOfChildren=s->numberOfChildrenList1();
         SgStatement * * children=new SgStatement * [org_numberOfChildren];

         int i;
         for(i=0;i<org_numberOfChildren;i++)
              children[i]=s->childList1(i);

         for(i=0;i<org_numberOfChildren;i++)
             analyse3(children[i]);

         delete [] children;
	   // 
	 replace_resultName(s);
         break;
      };  

  case FOR_NODE:
      {
	int org_numberOfChildren=s->numberOfChildrenList1();

	SgStatement * * children = new SgStatement * [org_numberOfChildren];

	int i;

	for (i=0; i<org_numberOfChildren; i++)
	    children[i]=s->childList1(i);

        for (i=0; i<org_numberOfChildren; i++)
  	    analyse3(children[i]);
      
        delete [] children;
	replace_resultName(s);
        break;
      }

  case LOGIF_NODE:
  case ELSEIF_NODE:
  case LOGIF_STMT:
  case SWITCH_NODE:
  case CASE_NODE:
    {
	int org_numberOfChildren1=s->numberOfChildrenList1();

	SgStatement * * children1 = new SgStatement * [org_numberOfChildren1];

	int org_numberOfChildren2=s->numberOfChildrenList2();

	SgStatement * * children2 = new SgStatement * [org_numberOfChildren2];

	int i;

	for (i=0; i<org_numberOfChildren1; i++)
	    children1[i]=s->childList1(i);

	for (i=0; i<org_numberOfChildren1; i++)
	    analyse3(children1[i]);
      
	delete [] children1;

        for (i=0; i<org_numberOfChildren2; i++)
	    children2[i]=s->childList2(i);			

        for (i=0; i<org_numberOfChildren2; i++)
	    analyse3(children2[i]);
      
        delete [] children2;
	// 
	replace_resultName(s);
        break;
    }

  case VAR_DECL:
  case ASSIGN_STAT:
  case COMBINED_DIRECTIVE_DECL:
  case DISTRIBUTE_DECL:
  case INHERIT_DECL:
  case DYNAMIC_DECL:
  case ALIGN_DECL:
  default:
	//
	replace_resultName(s);
	break;
  }
}

